PIPE_RESOURCES = "RESOURCES"
"""
Identifier of Resources Subservice for messages IO
"""
PIPE_EXECUTOR = "EXECUTOR"
"""
Identifier of Test-Suite Execution Subservice  for messages IO
"""


TESTER_ID_PATTERN = r"^[a-fA-F\d]{32}$"
